{% extends "smartmin/update.html" %}

{% load i18n %}

{% block pre-form %}
<div class="alert">
  {% blocktrans %}
  <strong>Your password has expired.</strong> Site policy states that you must pick a new password every {{ expire_days }} days, and that passwords must not have been used within the previous {{ window_days }} days.
  {% endblocktrans %}
</div>
{% endblock %}
