{% extends "smartmin/form.html" %}

{% load i18n %}

{% block pre-form %}
<div class="alert alert-info">{% trans "Please enter the email address you used to sign up and we will help you recover your password." %}</div>
{% endblock %}
