{% extends "smartmin/base.html" %}

{% load smartmin i18n %}

{% block content %}
{% block pjax %}

<div id="pjax" id="read-view">
  <div class="row">
    <div class="col-md-12 buttons">
      {% block read-buttons %}
      {% if view.edit_button %}
      <a class="btn btn-primary pull-right" href="../../update/{{ object.id }}/">{% trans "edit" %}</a>
      {% endif %}
      {% endblock %}
    </div>
  </div>

  <div class="row">
    <div class="col-md-12">
      {% block pre-fields %}
      {% endblock pre-fields %}

      {% block fields %}
      <table class="table table-striped">
        <tbody>
          {% for field in fields %}
          <tr class="read-{{field}}">
            <td class="read-label">{% get_label field %}</td>
            <td class="read-value">{% get_value object field %}&nbsp;</td>
          </tr>
          {% endfor %}

        {% block extra-fields %}
        {% endblock extra-fields %}
        </tbody>
      </table>
      {% endblock fields %}

      {% block post-fields %}
      {% endblock %}
    </div>
  </div>
</div>

{% endblock %}
{% endblock %}

