# Generated by Django 2.2.17 on 2021-02-19 14:03

from django.db import migrations


def delete_failed_logins(apps, schema_editor):  # pragma: no cover
    FailedLogin = apps.get_model("users", "FailedLogin")

    FailedLogin.objects.all().delete()


def noop(apps, schema_editor):  # pragma: no cover
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('users', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(delete_failed_logins, noop)

    ]
